#include "awoorwae.h"
#include <cmath>
#include <string>

namespace pabloader {

bool Awoorwae::OnUserCreate()
{
    if (!player->Load()) {
        std::cerr << "Could not load awoorwa.spr" << std::endl;
        return false;
    }

    SetPixelMode(olc::Pixel::MASK);

    return true;
}

bool Awoorwae::OnUserUpdate(float fElapsedTime)
{
    Clear(olc::BLACK);

    player->Update(fElapsedTime);
    player->Draw();
    return true;
}

Entity::Entity(Awoorwae* game_)
    : game(game_)
{
    xv = 0;
    yv = 30;
    x = game->ScreenWidth() / 2;
    y = game->ScreenHeight() / 2;
}

olc::rcode Entity::Load()
{
    return img.LoadFromPGESprFile("res/awoorwa.spr");
}

void Entity::Update(float dt)
{
    x += xv * dt;
    y += yv * dt;

    xv *= 1 - 0.2 * dt;

    if (!onGround) {
        yv += 50 * dt;
        yv *= 1 - 0.05 * dt;
    }

    transform.Reset();

    game->DrawString(1, 1, "A,D - move; SPACE - jump", olc::CYAN);

#ifdef _DEBUG
    game->DrawString(1, 10, std::to_string(x) + " " + std::to_string(y));
    game->DrawString(1, 18, std::to_string(xv) + " " + std::to_string(yv));
#endif
    bool userInput = false;

    if (game->GetKey(olc::SPACE).bHeld && onGround) {
        yv = -100;
        userInput = true;
    }

    if (game->GetKey(olc::A).bHeld) {
        xv -= 100 * dt;
        userInput = true;
    }

    if (game->GetKey(olc::D).bHeld) {
        xv += 100 * dt;
        userInput = true;
    }

    if (x > game->ScreenWidth() - img.width / 2) {
        xv *= -0.3;
        x = game->ScreenWidth() - img.width / 2;
    }

    if (x < img.width / 2) {
        xv *= -0.3;
        x = img.width / 2;
    }

    if (y > game->ScreenHeight() - img.height / 4) {
        yv *= -0.3;

        if (std::abs(yv) < 2) {
            yv = 0;
        }
        y = game->ScreenHeight() - img.height / 4;
    }

    onGround = y >= game->ScreenHeight() - img.height / 4;

    if (!userInput && std::abs(xv) < 2) {
        xv = 0;
    }
    if (xv < 0) {
        transform.Scale(-1, 1);
    }
    transform.Translate(x, y);
}

void Entity::Draw()
{
    if (std::abs(xv) < 1) {
        olc::GFX2D::DrawPartialSprite(&img, 0, img.height / 2, img.width, img.height / 2, transform, true);
    } else {
        olc::GFX2D::DrawPartialSprite(&img, 0, 0, img.width, img.height / 2, transform, true);
    }
#ifdef _DEBUG
    float x, y;
    transform.Forward(0, 0, x, y);
    game->Draw((int)x, (int)y, olc::RED);
#endif
}
}